use strict;
use warnings;

package Alt::Lexical::Var::ButSupportModernPerl;

our $AUTHORITY = 'cpan:TOBYINK';
our $VERSION   = '0.002';

1;

=pod

=encoding utf8

=head1 NAME

Alt::Lexical::Var::ButSupportModernPerl - alternative distribution of Lexical::Var, with support for more modern versions of Perl

=head1 SYNOPSIS

   use Lexical::Var;

=head1 DESCRIPTION

This is an alternative distribution of Lexical::Var, allowing use on Perl 5.21.7 and above. If you have had no problems installing Lexical::Var, then you do not need this module!

Hopefully an official release of Lexical::Var with support for modern Perl will happen, and I'll be able to remove this abomination from the CPAN. :-)

Version 0.002 of Alt::Lexical::Var::ButSupportModernPerl includes version 0.009 of Lexical::Var.

=head1 SEE ALSO

L<Alt>, L<Lexical::Var>.

=head1 COPYRIGHT, LICENCE, AND DISCLAIMER OF WARRANTIES

This file is distributed under the same conditions as L<Lexical::Var>.

=cut

